# Copyright (C) 2022 Sebastien Guerri
#
# This file is part of libtable.
#
# libtable is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# libtable is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from ._baseclass import BaseTableData
from .tableselectcontrol import TableSelectControl

from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.key_binding import merge_key_bindings


class TableEditControl(TableSelectControl):
    def __init__(self,
                 table: BaseTableData,
                 show_header: bool = None,
                 show_auto: bool = None,
                 global_key_bindings: bool = True,
                 show_search: bool = True,
                 show_sort: bool = False
                 ) -> None:
        self.kb = KeyBindings()
        super().__init__(table,
                         show_header=show_header,
                         show_auto=show_auto,
                         global_key_bindings=global_key_bindings,
                         show_search=show_search,
                         show_sort=show_sort
                         )

    def get_key_bindings(self) -> KeyBindings:
        key_bindings = super().get_key_bindings()
        return merge_key_bindings([self.kb, key_bindings])

    def addEvent(self, *name: str, fn) -> None:
        @self.kb.add(*name)
        def _(event):
            fn(self.get_current_index(), self.get_current_row()[1])



# Copyright (C) 2022 Sebastien Guerri
#
# This file is part of libtable.
#
# libtable is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# libtable is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from ._baseclass import BaseTableData as TableData
from ._baseclass import BaseTableHeader as TableHeader
from ._baseclass import BaseTableOptions as TableOptions
from .tablecontrol import TableControl
from .tableselectcontrol import TableSelectControl
from .tableeditcontrol import TableEditControl
from .table import Table
from .tableprompt import TablePrompt
from .tableselect import TableSelect
from .tabledit import TableEdit

__doc__ = """
libtable - A library for cli tables
===================================
"""

__all__ = [
    "TableData",
    "TableHeader",
    "TableOptions",
    "TableControl",
    "TableSelectControl",
    "TableEditControl",
    "Table",
    "TablePrompt",
    "TableSelect",
    "TableEdit"
]



# Copyright (C) 2022 Sebastien Guerri
#
# This file is part of libtable.
#
# libtable is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# libtable is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os

from ._baseclass import BaseTableData
from ._basetablecontrol import BaseTableControl

from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.filters import has_focus
from prompt_toolkit.filters.utils import to_filter
from prompt_toolkit.formatted_text import FormattedText


class TableSelectControl(BaseTableControl):
    def __init__(self,
                 table: BaseTableData,
                 show_header: bool = None,
                 show_auto: bool = None,
                 global_key_bindings: bool = True,
                 show_search: bool = True,
                 show_sort: bool = False
                 ) -> None:
        self.table = table
        self.width = os.get_terminal_size().columns
        self.cancelled = False
        self.show_search = show_search
        self.show_sort = show_sort
        self.searched = ""
        self.searched_update = False
        self.sort_methods = {}
        self.global_key_bindings = global_key_bindings
        self.__check(show_header, show_auto)
        super().__init__(self.table, self.width)

    def __check(self,
                show_header: bool,
                show_auto: bool
                ) -> None:
        if show_header is not None:
            self.table.options.show_header = show_header
        if show_auto is not None:
            self.table.options.show_auto = show_auto

    def _get_rows_tokens(self) -> [FormattedText([])]:
        rows = super()._get_rows_tokens()
        new_rows = []
        if self.show_search and self.searched != "":
            self.indexes.clear()
            new_index = 0
            old_index = 0
            for row in rows:
                is_filtered = False
                for (style, text) in row:
                    if self.searched.lower() in text.lower():
                        is_filtered = True
                if is_filtered:
                    new_rows.append(row)
                    self.indexes[new_index] = old_index
                    new_index += 1
                old_index += 1
        else:
            new_rows = rows
        return new_rows

    def _get_choice_tokens(self) -> FormattedText([]):
        tokens = super()._get_choice_tokens()
        if self.show_search and self.searched != "":
            tokens.append(("", "\n"))
            tokens.append(("bg:ansiyellow fg:ansiblack italic", f"> Searching: {self.searched} "))
        if self.show_search and self.searched_update:
            self.searched_update = False
            self.reset_selection()
        return tokens

    def get_key_bindings(self) -> KeyBindings:
        self.key_bindings = KeyBindings()

        @self.key_bindings.add('up', filter=has_focus(self))
        def _(event):
            self.up()

        @self.key_bindings.add('down', filter=has_focus(self))
        def _(event):
            self.down()

        @self.key_bindings.add('enter', filter=has_focus(self) & to_filter(self.global_key_bindings))
        def _(event):
            event.app.exit()

        @self.key_bindings.add('c-c', filter=has_focus(self) & to_filter(self.global_key_bindings))
        def _(event):
            self.cancelled = True
            event.app.exit()

        @self.key_bindings.add('c-s', '<any>', filter=has_focus(self) & to_filter(self.show_sort))
        def _(event):
            char = event.key_sequence[1].data
            if char not in '0123456789':
                return
            char = int(char)
            delta = 1 if self.table.options.show_auto else 0

            def sort(row):
                try:
                    index = char - delta
                    column = self.table.headers[index].column
                    return str(row[column])
                except Exception:
                    return ""

            if self.table.options.show_auto and char == 0:
                pass
            else:
                if (char - delta) in self.sort_methods.keys():
                    self.table.rows = sorted(self.table.rows, key=self.sort_methods[char - delta])
                else:
                    self.table.rows = sorted(self.table.rows, key=sort)

        @self.key_bindings.add('escape', filter=has_focus(self) & to_filter(self.show_search))
        def _(event):
            self.searched = ""
            self.searched_update = True

        @self.key_bindings.add('backspace', filter=has_focus(self) & to_filter(self.show_search))
        def _(event):
            self.searched = self.searched[:-1]
            self.searched_update = True

        @self.key_bindings.add('<any>', filter=has_focus(self) & to_filter(self.show_search))
        def _(event):
            char = event.key_sequence[0].data
            self.searched += char
            self.searched_update = True

        return self.key_bindings

    def add_sort_method(self, index: int, fn):
        self.sort_methods[index] = fn

    def get_response(self) -> tuple:
        if self.cancelled:
            return (-1, "Operation cancelled")
        else:
            return self.get_current_row()



# Copyright (C) 2022 Sebastien Guerri
#
# This file is part of libtable.
#
# libtable is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# libtable is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from ._baseclass import BaseTableData
from ._baseclass import BaseTableHeader

from prompt_toolkit.layout.controls import FormattedTextControl
from prompt_toolkit.formatted_text import to_formatted_text
from prompt_toolkit.formatted_text import HTML
from prompt_toolkit.formatted_text import FormattedText


class BaseTableControl(FormattedTextControl):
    def __init__(self,
                 table: BaseTableData,
                 width: int,
                 **kwargs
                 ) -> None:
        self.table = table
        self.width = width
        self.rows_count = len(self.table.rows)
        self.rows_count_width = len(str(len(self.table.rows))) + 1
        self.column_count = len(self.table.headers)
        self.column_widths: dict = {}
        self.column_align: dict = {}
        self.selected = 1 if self.table.options.show_header else 0
        self.indexes = {}
        self.__init_table()
        super().__init__(self._get_choice_tokens, show_cursor=False, **kwargs)

    def __init_table(self) -> None:
        self.column_widths.clear()
        nb_columns = 0
        available_width = self.width - (self.column_count - 1) * self.table.options.span
        available_width -= (self.rows_count_width + self.table.options.span) if self.table.options.show_auto else 0
        not_done = []
        index = 0
        header: BaseTableHeader
        for header in self.table.headers:
            if header.width != -1:
                available_width -= header.width
                self.column_widths[index] = header.width
            else:
                nb_columns += header.weight
                not_done.append((index, header.weight))
            self.column_align[index] = header.rightalign
            index += 1
        min_width = int(available_width / nb_columns)
        available_width -= min_width * nb_columns
        for (index, weight) in not_done:
            col_width = min_width * weight
            nb_span = min(weight, available_width)
            col_width += nb_span
            available_width -= nb_span
            self.column_widths[index] = col_width

    def _get__header_tokens(self) -> FormattedText([]):
        tokens = []
        if self.table.options.show_header:
            line = ""
            if self.table.options.show_auto:
                line += "N".rjust(self.rows_count_width)
                line += "".ljust(self.table.options.span)
            index = 0
            header: BaseTableHeader
            for header in self.table.headers:
                content = header.name[0:self.column_widths[index]]
                if self.column_align[index]:
                    content = content.rjust(self.column_widths[index])
                else:
                    content = content.ljust(self.column_widths[index])
                line += content
                if index != self.column_count - 1:
                    line += "".ljust(self.table.options.span)
                index += 1
            line += "\n"
            tokens.append((self.table.options.header_style, line))
        return tokens

    def _get_rows_tokens(self) -> [FormattedText([])]:
        self.indexes.clear()
        tokens = []
        row_index = 1
        row: dict
        for row in self.table.rows:
            formatted_line = FormattedText([])
            if self.table.options.show_auto:
                formatted_line.append(('', str(row_index).rjust(self.rows_count_width)))
                formatted_line.append(('', "".ljust(self.table.options.span)))
            index = 0

            header: BaseTableHeader
            for header in self.table.headers:
                available_width = self.column_widths[index]
                cell_items = [('', '')]
                if header.column in row.keys():
                    cell_items = to_formatted_text(HTML(row[header.column]))
                temp_items = []
                for cell_item in cell_items:
                    content = cell_item[1][0:available_width]
                    available_width -= len(content)
                    temp_items.append((cell_item[0], content))
                if available_width > 0 and self.column_align[index]:
                    formatted_line.append(('', "".rjust(available_width)))
                formatted_line.extend(temp_items)
                if available_width > 0 and not self.column_align[index]:
                    formatted_line.append(('', "".rjust(available_width)))
                if index != self.column_count - 1:
                    formatted_line.append(('', "".ljust(self.table.options.span)))
                index += 1

            tokens.append(formatted_line)
            self.indexes[row_index - 1] = row_index - 1
            row_index += 1
        return tokens

    def __get_rows_tokens_with_style(self) -> FormattedText([]):
        rows = self._get_rows_tokens()
        self.rows_count = len(rows)
        tokens = []
        even = True
        row_index = 1
        for row in rows:
            style = "bg:#222222" if self.table.options.show_even and even else ""
            delta = 0 if self.table.options.show_header else 1
            style = self.table.options.selection_style if self.table.options.show_selection and row_index == self.selected + delta else style
            if row_index != self.rows_count:
                row.append(('', "\n"))
            formatted_line = to_formatted_text(row, style=style)
            tokens.append(formatted_line)
            row_index += 1
            even = not even
        if self.rows_count == 0:
            tokens.append([("italic fg:grey", "No data")])
        return tokens

    def _get_choice_tokens(self) -> FormattedText([]):
        tokens = []
        if self.table.options.show_header:
            tokens.extend(self._get__header_tokens())
        for row in self.__get_rows_tokens_with_style():
            tokens.extend(row)
        return tokens

    def up(self) -> None:
        min_index = 1 if self.table.options.show_header else 0
        if self.selected != min_index:
            self.selected -= 1

    def down(self) -> None:
        max_delta = 0 if self.table.options.show_header else 1
        if self.selected != self.rows_count - max_delta:
            self.selected += 1

    def get_selection(self) -> int:
        if self.selected == -1:
            return -1
        delta = 1 if self.table.options.show_header else 0
        return self.selected - delta

    def clear_selection(self) -> None:
        self.selected = -1

    def reset_selection(self) -> None:
        self.selected = 1 if self.table.options.show_header else 0

    def set_selection(self, value) -> None:
        min_index = 1 if self.table.options.show_header else 0
        max_index = self.rows_count - (1 - min_index)
        self.selected = value
        self.selected = max(min_index, self.selected)
        self.selected = min(max_index, self.selected)

    def get_current_index(self) -> int:
        try:
            return self.indexes[self.get_selection()]
        except Exception:
            return -1

    def get_current_row(self) -> tuple:
        index = self.get_current_index()
        if index == -1:
            return (-1, 'No selection')
        return (index, self.table.rows[index])

    def to_formatted_text(self) -> FormattedText:
        return FormattedText(self._get_choice_tokens())



# Copyright (C) 2022 Sebastien Guerri
#
# This file is part of libtable.
#
# libtable is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# libtable is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os

from ._baseclass import BaseTableData
from ._basetablecontrol import BaseTableControl


class TableControl(BaseTableControl):
    def __init__(self,
                 table: BaseTableData,
                 show_header: bool = None,
                 show_auto: bool = None
                 ) -> None:
        self.table = table
        self.width = os.get_terminal_size().columns
        self.__check(show_header, show_auto)
        super().__init__(self.table, self.width)
        self.clear_selection()

    def __check(self,
                show_header: bool,
                show_auto: bool
                ) -> None:
        if show_header is not None:
            self.table.options.show_header = show_header
        if show_auto is not None:
            self.table.options.show_auto = show_auto



# Copyright (C) 2022 Sebastien Guerri
#
# This file is part of libtable.
#
# libtable is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# libtable is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from ._baseclass import BaseTableData
from .tableselectcontrol import TableSelectControl

from prompt_toolkit import Application
from prompt_toolkit.layout.layout import Layout
from prompt_toolkit.layout.containers import HSplit
from prompt_toolkit.layout.containers import Window


class TableSelect:
    def __init__(self,
                 table: BaseTableData,
                 full_screen: bool = False,
                 erase_when_done: bool = False,
                 show_header: bool = None,
                 show_auto: bool = None,
                 show_search: bool = True,
                 show_sort: bool = True
                 ) -> None:
        self.table = table
        self.full_screen = full_screen
        self.erase_when_done = erase_when_done
        self.table_control = TableSelectControl(self.table,
                                                show_header=show_header,
                                                show_auto=show_auto,
                                                show_search=show_search,
                                                show_sort=show_sort
                                                )

    def show(self) -> tuple:
        body = HSplit([
            Window(content=self.table_control)
        ])
        self.app = Application(layout=Layout(body),
                               full_screen=self.full_screen,
                               key_bindings=self.table_control.get_key_bindings(),
                               erase_when_done=self.erase_when_done
                               )
        self.app.run()
        return self.table_control.get_response()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['libtable']

package_data = \
{'': ['*']}

install_requires = \
['prompt-toolkit>=3.0.29,<4.0.0']

setup_kwargs = {
    'name': 'libtable',
    'version': '0.3.0',
    'description': 'Python library for cli tables',
    'long_description': '# Welcome to libtable\n\n[![](https://badgen.net/github/release/sguerri/libtable)](https://github.com/sguerri/libtable/releases/)\n[![](https://img.shields.io/github/workflow/status/sguerri/libtable/Build/v0.3.0)](https://github.com/sguerri/libtable/actions/workflows/build.yml)\n[![](https://badgen.net/github/license/sguerri/libtable)](https://www.gnu.org/licenses/)\n[![](https://badgen.net/pypi/v/libtable)](https://pypi.org/project/libtable/)\n[![](https://badgen.net/pypi/python/libtable)](#)\n[![](https://badgen.net/badge/Open%20Source%20%3F/Yes%21/blue?icon=github)](#)\n\n> Python library for cli tables\n\nWork in Progress\n\n**Main features**\n* TODO\n\n**Roadmap**\n* TODO\n\n---\n\n- [Welcome to libtable](#welcome-to-libtable)\n  * [Installation](#installation)\n  * [Usage](#usage)\n  * [Build](#build)\n  * [Dependencies](#dependencies)\n  * [Author](#author)\n  * [Issues](#issues)\n  * [License](#license)\n\n## Installation\n\nTODO\n\n## Usage\n\nTODO\n\n## Build\n\nTODO\n\n## Dependencies\n\n**Python Libraries**\n- [python-prompt-toolkit](https://github.com/prompt-toolkit/python-prompt-toolkit)\n\n**Python Development Libraries**\n- [poetry](https://python-poetry.org/)\n\n## Author\n\nSébastien Guerri - [github page](https://github.com/sguerri)\n\n## Issues\n\nContributions, issues and feature requests are welcome!\n\nFeel free to check [issues page](https://github.com/sguerri/libtable/issues). You can also contact me.\n\n## License\n\nCopyright (C) 2022 Sebastien Guerri\n\nlibtable is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.\n\nlibtable is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License along with libtable. If not, see <https://www.gnu.org/licenses/>.',
    'author': 'Sebastien GUERRI',
    'author_email': 'nierrgu@bmel.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/sguerri/libtable',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0',
}


setup(**setup_kwargs)



# Copyright (C) 2022 Sebastien Guerri
#
# This file is part of libtable.
#
# libtable is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# libtable is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import json


class BaseTableHeader:
    def __init__(self,
                 column: str,
                 name: str,
                 width: int = -1,
                 weight: int = 1,
                 rightalign: bool = False
                 ) -> None:
        self.column = column
        self.name = name
        self.width = width
        self.weight = weight
        self.rightalign = rightalign

    def to_json(self) -> json:
        return {
            "column": self.column,
            "name": self.name,
            "width": self.width,
            "weight": self.weight,
            "rightalign": self.rightalign
        }

    def __str__(self) -> str:
        return json.dumps(self.to_json(), indent=4)


class BaseTableOptions():
    def __init__(self,
                 span: int = 1,
                 show_auto: bool = False,
                 show_even: bool = True,
                 show_header: bool = True,
                 show_selection: bool = True,
                 header_style: str = "bold ansimagenta underline",
                 selection_style: str = "bg:ansiblue"
                 ) -> None:
        self.span = span
        self.show_auto = show_auto
        self.show_even = show_even
        self.show_header = show_header
        self.show_selection = show_selection
        self.header_style = header_style
        self.selection_style = selection_style

    def to_json(self) -> json:
        return {
            "span": self.span,
            "show_auto": self.show_auto,
            "show_even": self.show_even,
            "show_header": self.show_header,
            "show_selection": self.show_selection,
            "header_style": self.header_style,
            "selection_style": self.selection_style
        }

    def __str__(self) -> str:
        return json.dumps(self.to_json(), indent=4)


class BaseTableData:
    def __init__(self,
                 headers: [BaseTableHeader] = [],
                 rows: [dict] = [],
                 options: BaseTableOptions = BaseTableOptions()
                 ) -> None:
        self.headers: [BaseTableHeader] = headers
        self.rows = rows
        self.options = options

    def add_header(self,
                   column: str,
                   name: str,
                   width: int = -1,
                   weight: int = 1,
                   rightalign: bool = False
                   ):
        self.headers.append(BaseTableHeader(column, name, width=width, weight=weight, rightalign=rightalign))

    def to_json(self) -> json:
        return {
            "headers": list(map(lambda header: header.to_json(), self.headers)),
            "rows": json.loads(self.rows),
            "options": self.options.to_json()
        }

    def __str__(self) -> str:
        return json.dumps(self.to_json(), indent=4)



# Copyright (C) 2022 Sebastien Guerri
#
# This file is part of libtable.
#
# libtable is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# libtable is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from ._baseclass import BaseTableData
from .tablecontrol import TableControl
from prompt_toolkit import print_formatted_text


class Table:
    def __init__(self,
                 table: BaseTableData,
                 show_header: bool = None,
                 show_auto: bool = None
                 ):
        self.table = table
        self.table_control = TableControl(table, show_header=show_header, show_auto=show_auto)

    def show(self):
        print_formatted_text(self.table_control.to_formatted_text())



# Copyright (C) 2022 Sebastien Guerri
#
# This file is part of libtable.
#
# libtable is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# libtable is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from ._baseclass import BaseTableData
from .tablecontrol import TableControl

from prompt_toolkit import prompt
from prompt_toolkit import print_formatted_text
from prompt_toolkit.formatted_text import FormattedText
from prompt_toolkit.validation import Validator


class TablePrompt:
    def __init__(self,
                 table: BaseTableData,
                 show_header: bool = None,
                 show_auto: bool = None,
                 index_column: str = "auto",
                 prompt_style: str = "ansiyellow"
                 ):
        self.table = table
        self.index_column = index_column
        self.prompt_style = prompt_style
        self.valid = True
        self.error_message = ""
        self.__get_valid_values()
        self.table_control = TableControl(table, show_header=show_header, show_auto=show_auto)
        self.table_control.clear_selection()
        self.validator = Validator.from_callable(
            self.__is_valid_index,
            error_message=f"Invalid input ({self.index_column} value expected)",
            move_cursor_to_end=True,
        )

    def __get_valid_values(self):
        self.valid_values = []
        if self.index_column == "auto":
            if not self.table.options.show_auto:
                self.valid = False
                self.error_message = "ERROR: index_column is auto but no auto column"
            else:
                self.valid_values = list(range(1, len(self.table.rows) + 1))
        else:
            headers = list(map(lambda x: x.column, self.table.headers))
            if self.index_column not in headers:
                self.valid = False
                self.error_message = "ERROR: index_column is not a valid column"
            else:
                # self.column_index = headers.index(self.index_column)
                self.valid_values = list(map(lambda x: x[self.index_column] if self.index_column in x.keys() else None, self.table.rows))
                self.valid_values = list(filter(None, self.valid_values))
                self.valid_values = list(filter(lambda x: str(x).isnumeric(), self.valid_values))
                if len(self.valid_values) == 0:
                    self.valid = False
                    self.error_message = "ERROR: no valid values"
                self.valid_values = list(map(lambda x: int(x), self.valid_values))

    def __is_valid_index(self, text):
        try:
            index = int(text)
        except ValueError:
            return False
        return index in self.valid_values

    def show(self):
        if not self.valid:
            return (-1, self.error_message)
        print_formatted_text(FormattedText(self.table_control._get_choice_tokens()))
        try:
            message_part = "index" if self.index_column == "auto" else self.index_column
            index = int(prompt(FormattedText([(self.prompt_style, f"Enter {message_part}: ")]), validator=self.validator, validate_while_typing=False))
        except KeyboardInterrupt:
            return (-1, "Operation cancelled")
        if self.index_column == "auto":
            return (index - 1, self.table.rows[index - 1])
        else:
            return (index, list(filter(lambda x: int(x[self.index_column]) == index, self.table.rows))[0])
